import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintVarsListAppend(TestBaseClass):

    @pytest.mark.parametrize('id_', ['oelint.vars.listappend'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'PACKAGES =. "foo"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'FILES:${PN}-tracepath:append = "${base_bindir}"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'RDEPENDS:${PN} .= "bar"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'DEPENDS:prepend = "xyz"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI .= "file://abc"',
                                 },
                             ],
                             )
    def test_bad(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.listappend'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'PACKAGES =. "foo"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'FILES_${PN}-tracepath_append = "${base_bindir}"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'RDEPENDS_${PN} .= "bar"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'DEPENDS_prepend = "xyz"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI .= "file://abc"',
                                 },
                             ],
                             )
    def test_bad_old(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_, ['--release=dunfell']), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.listappend'])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'PACKAGES =. "foo"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'FILES:${PN}-tracepath:append = "${base_bindir}"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'RDEPENDS:${PN} .= "bar"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'DEPENDS:prepend = "xyz"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI .= "file://abc"',
                                 },
                             ],
                             )
    def test_fix(self, input_, id_):
        self.fix_and_check(self._create_args_fix(input_), id_)

    @pytest.mark.parametrize('id_', ['oelint.vars.listappend'])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'PACKAGES =. "foo"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'FILES_${PN}-tracepath_append = "${base_bindir}"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'RDEPENDS_${PN} .= "bar"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'DEPENDS_prepend = "xyz"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI .= "file://abc"',
                                 },
                             ],
                             )
    def test_fix_old(self, input_, id_):
        self.fix_and_check(self._create_args_fix(input_, ['--release=dunfell']), id_)

    @pytest.mark.parametrize('id_', ['oelint.vars.listappend'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'PACKAGES =. "foo "',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'FILES:${PN}-tracepath:append = " ${base_bindir}"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'RDEPENDS:${PN} .= " bar"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'DEPENDS:prepend = "xyz "',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI .= " file://abc"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'FILESPATH:prepend := "${THISDIR}/file:"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'FILESEXTRAPATHS:prepend := "${THISDIR}/file:"',
                                 },
                             ],
                             )
    def test_good(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.listappend'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'PACKAGES =. "foo "',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'FILES_${PN}-tracepath_append = " ${base_bindir}"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'RDEPENDS_${PN} .= " bar"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'DEPENDS_prepend = "xyz "',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI .= " file://abc"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'FILESPATH_prepend := "${THISDIR}/file:"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'FILESEXTRAPATHS_prepend := "${THISDIR}/file:"',
                                 },
                             ],
                             )
    def test_good_old(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_, ['--release=dunfell']), id_, occurrence)



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintVarsSRCURIfile(TestBaseClass):

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurifile'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     SRC_URI += "file://abc"
                                     SRC_URI += "git://foo.org/gaz.git"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     SRC_URI += "file://abc"
                                     SRC_URI += "http://foo.org/abc.tar.gz"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     SRC_URI += "file://abc"
                                     inherit foo
                                     SRC_URI += "http://foo.org/abc.tar.gz"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     SRC_URI += "http://def.com/bar.zip"
                                     SRC_URI:prepend = "file://abc"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     SRC_URI:append = "file://abc"
                                     SRC_URI:append = "http://def.com/bar.zip"
                                     ''',
                                 },
                             ],
                             )
    def test_bad(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurifile'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI += "${@do_magic_function(d)} file://abc"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     SRC_URI += "file://abc"
                                     SRC_URI += "file://def"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI += "file://abc"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI += "file://abc"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI = "file://foo ${@bb.utils.contains(\'FOO\', \'bar\', \'file://bar\', \'\', d)}"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI[sha256sum] = "file://abc"',
                                 },
                                 {
                                     'recipes/oelint_adv_test.bb':
                                     '''
                                     A = "1"
                                     inherit foo
                                     ''',
                                     'conf/layer.conf':
                                     ' ',
                                     'classes/foo.bbclass':
                                     'SRC_URI ?= "https://some.corp.org/${PN}"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     SRC_URI:append = "file://abc"
                                     SRC_URI += "http://def.com/bar.zip"
                                     ''',
                                 },
                             ],
                             )
    def test_good(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)



import pytest  # noqa: I900

from .base import TestBaseClass

# flake8: noqa W293 - we want to explicitly test lines with just whitespaces here

class TestClassOelintTaskAddNoBody(TestBaseClass):

    @pytest.mark.parametrize('id_', ['oelint.task.addnotaskbody'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     do_foo() {
                                     
                                     }
                                     addtask do_foo
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     addtask do_notexists
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     do_foo() {
                                     
                                     }
                                     addtask foo
                                     ''',
                                 },
                             ],
                             )
    def test_bad(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.task.addnotaskbody'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     do_foo() {
                                         :
                                     }
                                     addtask do_foo
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     do_foo() {
                                         :
                                     }
                                     addtask foo
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     addtask build
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     addtask configure
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     addtask compile
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     addtask install
                                     ''',
                                 },
                             ],
                             )
    def test_good(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)



import pytest  # noqa: I900
from .base import TestBaseClass


class TestClassOelintJetmDependsSingleLine(TestBaseClass):

    @pytest.mark.parametrize('id_', ['oelint.jetm.vars.dependssingleline'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'DEPENDS += "abc def"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     DEPENDS += "ghi \\
                                         jkl"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'RDEPENDS:${PN} += "abc def"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     RDEPENDS:${PN} += "ghi \\
                                         jkl"
                                     ''',
                                 },
                             ],
                             )
    def test_bad(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_, extraopts=[
                          '--addrules=jetm']), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.jetm.vars.dependssingleline'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'RDEPENDS_${PN} += "abc def"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     RDEPENDS_${PN} += "ghi \\
                                         jkl"
                                     ''',
                                 },
                             ],
                             )
    def test_bad_old(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_, extraopts=[
                          '--addrules=jetm', '--release=dunfell']), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.jetm.vars.dependssingleline'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     DEPENDS += "abc"
                                     DEPENDS += "def"
                                     DEPENDS += "ghi"
                                     DEPENDS += "jkl"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     RDEPENDS:${PN} += "abc"
                                     RDEPENDS:${PN} += "def"
                                     RDEPENDS:${PN} += "ghi"
                                     RDEPENDS:${PN} += "jkl"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     DEPENDS += "abc (>= 1.2.3)"
                                     DEPENDS += "def (<= 4.5.6)"
                                     ''',
                                 },
                             ],
                             )
    def test_good(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_, extraopts=[
                          '--addrules=jetm']), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.jetm.vars.dependssingleline'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     RDEPENDS_${PN} += "abc"
                                     RDEPENDS_${PN} += "def"
                                     RDEPENDS_${PN} += "ghi"
                                     RDEPENDS_${PN} += "jkl"
                                     ''',
                                 },
                             ],
                             )
    def test_good_old(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_, extraopts=[
                          '--addrules=jetm', '--release=dunfell']), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.jetm.vars.dependssingleline'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'DEPENDS += "abc def"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     DEPENDS += "ghi \\
                                         jkl"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'RDEPENDS:${PN} += "abc def"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     RDEPENDS:${PN} += "ghi \\
                                         jkl"
                                     ''',
                                 },
                             ],
                             )
    def test_good_module_off(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.jetm.vars.dependssingleline'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'DEPENDS += "abc def"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     DEPENDS += "ghi \\
                                         jkl"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'RDEPENDS_${PN} += "abc def"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     RDEPENDS_${PN} += "ghi \\
                                         jkl"
                                     ''',
                                 },
                             ],
                             )
    def test_good_module_off_old(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_, ['--release=dunfell']), id_, occurrence)



import pytest  # noqa: I900

from .base import TestBaseClass

OPTIONS_AVAILABLE = [
    'apply',
    'branch',
    'date',
    'destsuffix',
    'dev',
    'downloadfilename',
    'fullpath',
    'localdir',
    'manifest',
    'method',
    'module',
    'name',
    'nobranch',
    'nocheckout',
    'norecurse',
    'noverify',
    'patchdir',
    'path_spec',
    'port',
    'proto',
    'protocol',
    'rebaseable',
    'rev',
    'revision',
    'rsh',
    'scmdata',
    'ssh',
    'striplevel',
    'subdir',
    'subpath',
    'tag',
    'transportuser',
    'unpack',
    'usehead',
    'version',
    'vob',
    'unknownoperation',
]
OPTION_MAPPING = {
    'az': [
        'apply',
        'destsuffix',
        'name',
        'patchdir',
        'striplevel',
        'subdir',
        'unpack',
    ],
    'bzr': [
        'apply',
        'destsuffix',
        'name',
        'patchdir',
        'protocol',
        'scmdata',
        'striplevel',
        'subdir',
        'unpack',
    ],
    'crate': [
        'apply',
        'destsuffix',
        'downloadfilename',
        'name',
        'patchdir',
        'striplevel',
        'subdir',
        'unpack',
    ],
    'crcc': [
        'apply',
        'destsuffix',
        'module',
        'name',
        'patchdir',
        'protocol',
        'striplevel',
        'subdir',
        'unpack',
        'vob',
    ],
    'cvs': [
        'apply',
        'date',
        'destsuffix',
        'fullpath',
        'localdir',
        'method',
        'module',
        'name',
        'norecurse',
        'patchdir',
        'port',
        'rsh',
        'scmdata',
        'striplevel',
        'subdir',
        'tag',
        'unpack',
    ],
    'file': [
        'apply',
        'destsuffix',
        'downloadfilename',
        'name',
        'patchdir',
        'striplevel',
        'subdir',
        'unpack',
    ],
    'ftp': [
        'apply',
        'destsuffix',
        'downloadfilename',
        'name',
        'patchdir',
        'striplevel',
        'subdir',
        'unpack',
    ],
    'gs': [
        'downloadfilename',
    ],
    'git': [
        'apply',
        'branch',
        'destsuffix',
        'name',
        'nobranch',
        'nocheckout',
        'patchdir',
        'protocol',
        'rebaseable',
        'rev',
        'striplevel',
        'subdir',
        'subpath',
        'tag',
        'unpack',
        'usehead',
        'user',
    ],
    'gitsm': [
        'apply',
        'branch',
        'destsuffix',
        'name',
        'nobranch',
        'nocheckout',
        'patchdir',
        'protocol',
        'rebaseable',
        'rev',
        'striplevel',
        'subdir',
        'subpath',
        'tag',
        'unpack',
        'usehead',
    ],
    'gitannex': [
        'apply',
        'destsuffix',
        'name',
        'patchdir',
        'striplevel',
        'subdir',
        'unpack',
    ],
    'gn': [
        'destdir',
        'name',
        'proto',
    ],
    'gomod': [
        'downloadfilename',
        'mod',
        'module',
        'version',
    ],
    'gomodgit': [
        'bareclone',
        'branch',
        'module',
        'nobranch',
        'protocol',
        'repo',
        'srcrev',
        'version',
    ],
    'hg': [
        'apply',
        'destsuffix',
        'module',
        'name',
        'patchdir',
        'protocol',
        'rev',
        'scmdata',
        'striplevel',
        'subdir',
        'unpack',
    ],
    'http': [
        'apply',
        'destsuffix',
        'downloadfilename',
        'name',
        'patchdir',
        'striplevel',
        'subdir',
        'unpack',
    ],
    'https': [
        'apply',
        'destsuffix',
        'downloadfilename',
        'name',
        'patchdir',
        'striplevel',
        'subdir',
        'unpack',
    ],
    'osc': [
        'apply',
        'destsuffix',
        'module',
        'name',
        'patchdir',
        'protocol',
        'rev',
        'striplevel',
        'subdir',
        'unpack',
    ],
    'p4': [
        'apply',
        'destsuffix',
        'name',
        'patchdir',
        'revision',
        'striplevel',
        'subdir',
        'unpack',
    ],
    'repo': [
        'apply',
        'branch',
        'destsuffix',
        'manifest',
        'name',
        'patchdir',
        'protocol',
        'striplevel',
        'subdir',
        'unpack',
    ],
    'ssh': [
        'apply',
        'destsuffix',
        'name',
        'patchdir',
        'striplevel',
        'subdir',
        'unpack',
    ],
    's3': [
        'apply',
        'destsuffix',
        'downloadfilename',
        'name',
        'patchdir',
        'striplevel',
        'subdir',
        'unpack',
    ],
    'sftp': [
        'apply',
        'destsuffix',
        'downloadfilename',
        'name',
        'patchdir',
        'protocol',
        'striplevel',
        'subdir',
        'unpack',
    ],
    'npm': [
        'apply',
        'destsuffix',
        'downloadfilename',
        'name',
        'noverify',
        'package',
        'patchdir',
        'striplevel',
        'subdir',
        'unpack',
        'version',
    ],
    'npmsw': [
        'apply',
        'destsuffix',
        'dev',
        'downloadfilename',
        'name',
        'patchdir',
        'protocol',
        'rev',
        'striplevel',
        'subdir',
        'unpack',
    ],
    'svn': [
        'apply',
        'destsuffix',
        'module',
        'name',
        'patchdir',
        'path_spec',
        'protocol',
        'rev',
        'scmdata',
        'ssh',
        'striplevel',
        'subdir',
        'transportuser',
        'unpack',
    ],
}


class TestClassOelintVarsSRCURIOptions(TestBaseClass):

    def __generate_sample_code(self, protocol, option):
        if not option:
            return '''
                SRC_URI += "{protocol}://foo"
                '''.format(protocol=protocol)
        return '''
            SRC_URI += "{protocol}://foo;{option}=1"
            '''.format(protocol=protocol, option=option)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('protocol', ['az'])
    @pytest.mark.parametrize('option', [x for x in OPTIONS_AVAILABLE if x not in OPTION_MAPPING['az']])
    def test_bad_az(self, id_, occurrence, protocol, option):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(protocol, option),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('protocol', ['az'])
    @pytest.mark.parametrize('option', OPTION_MAPPING['az'])
    def test_good_az(self, id_, occurrence, protocol, option):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(protocol, option),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('protocol', ['bzr'])
    @pytest.mark.parametrize('option', [x for x in OPTIONS_AVAILABLE if x not in OPTION_MAPPING['bzr']])
    def test_bad_bzr(self, id_, occurrence, protocol, option):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(protocol, option),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('protocol', ['bzr'])
    @pytest.mark.parametrize('option', OPTION_MAPPING['bzr'])
    def test_good_bzr(self, id_, occurrence, protocol, option):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(protocol, option),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('protocol', ['crate'])
    @pytest.mark.parametrize('option', [x for x in OPTIONS_AVAILABLE if x not in OPTION_MAPPING['crate']])
    def test_bad_crate(self, id_, occurrence, protocol, option):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(protocol, option),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('protocol', ['crate'])
    @pytest.mark.parametrize('option', OPTION_MAPPING['crate'])
    def test_good_crate(self, id_, occurrence, protocol, option):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(protocol, option),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('protocol', ['crcc'])
    @pytest.mark.parametrize('option', [x for x in OPTIONS_AVAILABLE if x not in OPTION_MAPPING['crcc']])
    def test_bad_crcc(self, id_, occurrence, protocol, option):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(protocol, option),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('protocol', ['crcc'])
    @pytest.mark.parametrize('option', OPTION_MAPPING['crcc'])
    def test_good_crcc(self, id_, occurrence, protocol, option):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(protocol, option),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('protocol', ['cvs'])
    @pytest.mark.parametrize('option', [x for x in OPTIONS_AVAILABLE if x not in OPTION_MAPPING['cvs']])
    def test_bad_cvs(self, id_, occurrence, protocol, option):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(protocol, option),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('protocol', ['cvs'])
    @pytest.mark.parametrize('option', OPTION_MAPPING['cvs'])
    def test_good_cvs(self, id_, occurrence, protocol, option):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(protocol, option),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('protocol', ['file'])
    @pytest.mark.parametrize('option', [x for x in OPTIONS_AVAILABLE if x not in OPTION_MAPPING['file']])
    def test_bad_file(self, id_, occurrence, protocol, option):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(protocol, option),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('protocol', ['file'])
    @pytest.mark.parametrize('option', OPTION_MAPPING['file'])
    def test_good_file(self, id_, occurrence, protocol, option):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(protocol, option),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('protocol', ['ftp'])
    @pytest.mark.parametrize('option', [x for x in OPTIONS_AVAILABLE if x not in OPTION_MAPPING['ftp']])
    def test_bad_ftp(self, id_, occurrence, protocol, option):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(protocol, option),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('protocol', ['ftp'])
    @pytest.mark.parametrize('option', OPTION_MAPPING['ftp'])
    def test_good_ftp(self, id_, occurrence, protocol, option):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(protocol, option),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('protocol', ['gs'])
    @pytest.mark.parametrize('option', ['foo', 'bar', 'baz'])
    def test_bad_gcp(self, id_, occurrence, protocol, option):
        input_ = {'oelint_adv_test.bb': self.__generate_sample_code(protocol, option)}
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('protocol', ['gs'])
    @pytest.mark.parametrize('option', OPTION_MAPPING['gs'])
    def test_good_gcp(self, id_, occurrence, protocol, option):
        input_ = {'oelint_adv_test.bb': self.__generate_sample_code(protocol, option)}
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('protocol', ['git'])
    @pytest.mark.parametrize('option', [x for x in OPTIONS_AVAILABLE if x not in OPTION_MAPPING['git']])
    def test_bad_git(self, id_, occurrence, protocol, option):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(protocol, 'protocol=ssh;nobranch=1;' + option),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('protocol', ['git'])
    @pytest.mark.parametrize('option', OPTION_MAPPING['git'])
    def test_good_git(self, id_, occurrence, protocol, option):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(protocol, 'protocol=ssh;nobranch=1;' + option),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('protocol', ['gitsm'])
    @pytest.mark.parametrize('option', [x for x in OPTIONS_AVAILABLE if x not in OPTION_MAPPING['gitsm']])
    def test_bad_gitsm(self, id_, occurrence, protocol, option):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(protocol, 'protocol=ssh;nobranch=1;' + option),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('protocol', ['gitsm'])
    @pytest.mark.parametrize('option', OPTION_MAPPING['gitsm'])
    def test_good_gitsm(self, id_, occurrence, protocol, option):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(protocol, 'protocol=ssh;nobranch=1;' + option),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('protocol', ['gitannex'])
    @pytest.mark.parametrize('option', [x for x in OPTIONS_AVAILABLE if x not in OPTION_MAPPING['gitannex']])
    def test_bad_gitannex(self, id_, occurrence, protocol, option):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(protocol, option),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('protocol', ['gitannex'])
    @pytest.mark.parametrize('option', OPTION_MAPPING['gitannex'])
    def test_good_gitannex(self, id_, occurrence, protocol, option):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(protocol, option),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('protocol', ['gomod'])
    @pytest.mark.parametrize('option', ['xyz', 'abc'])
    def test_bad_gomod(self, id_, occurrence, protocol, option):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(protocol, 'version=1;' + option),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('protocol', ['gomod'])
    @pytest.mark.parametrize('option', OPTION_MAPPING['gomod'])
    def test_good_gomod(self, id_, occurrence, protocol, option):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(protocol, 'version=1;' + option),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('protocol', ['gomodgit'])
    @pytest.mark.parametrize('option', ['xyz', 'abc'])
    def test_bad_gomodgit(self, id_, occurrence, protocol, option):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(protocol, 'version=1;' + option),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('protocol', ['gomodgit'])
    @pytest.mark.parametrize('option', OPTION_MAPPING['gomodgit'])
    def test_good_gomodgit(self, id_, occurrence, protocol, option):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(protocol, 'version=1;' + option),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('protocol', ['hg'])
    @pytest.mark.parametrize('option', [x for x in OPTIONS_AVAILABLE if x not in OPTION_MAPPING['hg']])
    def test_bad_hg(self, id_, occurrence, protocol, option):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(protocol, option),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('protocol', ['hg'])
    @pytest.mark.parametrize('option', OPTION_MAPPING['hg'])
    def test_good_hg(self, id_, occurrence, protocol, option):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(protocol, option),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('protocol', ['http'])
    @pytest.mark.parametrize('option', [x for x in OPTIONS_AVAILABLE if x not in OPTION_MAPPING['http']])
    def test_bad_http(self, id_, occurrence, protocol, option):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(protocol, option),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('protocol', ['http'])
    @pytest.mark.parametrize('option', OPTION_MAPPING['http'])
    def test_good_http(self, id_, occurrence, protocol, option):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(protocol, option),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('protocol', ['https'])
    @pytest.mark.parametrize('option', [x for x in OPTIONS_AVAILABLE if x not in OPTION_MAPPING['https']])
    def test_bad_https(self, id_, occurrence, protocol, option):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(protocol, option),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('protocol', ['https'])
    @pytest.mark.parametrize('option', OPTION_MAPPING['https'])
    def test_good_https(self, id_, occurrence, protocol, option):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(protocol, option),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('protocol', ['osc'])
    @pytest.mark.parametrize('option', [x for x in OPTIONS_AVAILABLE if x not in OPTION_MAPPING['osc']])
    def test_bad_osc(self, id_, occurrence, protocol, option):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(protocol, option),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('protocol', ['osc'])
    @pytest.mark.parametrize('option', OPTION_MAPPING['osc'])
    def test_good_osc(self, id_, occurrence, protocol, option):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(protocol, option),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('protocol', ['p4'])
    @pytest.mark.parametrize('option', [x for x in OPTIONS_AVAILABLE if x not in OPTION_MAPPING['p4']])
    def test_bad_p4(self, id_, occurrence, protocol, option):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(protocol, option),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('protocol', ['p4'])
    @pytest.mark.parametrize('option', OPTION_MAPPING['p4'])
    def test_good_p4(self, id_, occurrence, protocol, option):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(protocol, option),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('protocol', ['repo'])
    @pytest.mark.parametrize('option', [x for x in OPTIONS_AVAILABLE if x not in OPTION_MAPPING['repo']])
    def test_bad_repo(self, id_, occurrence, protocol, option):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(protocol, option),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('protocol', ['repo'])
    @pytest.mark.parametrize('option', OPTION_MAPPING['repo'])
    def test_good_repo(self, id_, occurrence, protocol, option):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(protocol, option),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('protocol', ['ssh'])
    @pytest.mark.parametrize('option', [x for x in OPTIONS_AVAILABLE if x not in OPTION_MAPPING['ssh']])
    def test_bad_ssh(self, id_, occurrence, protocol, option):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(protocol, option),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('protocol', ['ssh'])
    @pytest.mark.parametrize('option', OPTION_MAPPING['ssh'])
    def test_good_ssh(self, id_, occurrence, protocol, option):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(protocol, option),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('protocol', ['s3'])
    @pytest.mark.parametrize('option', [x for x in OPTIONS_AVAILABLE if x not in OPTION_MAPPING['s3']])
    def test_bad_s3(self, id_, occurrence, protocol, option):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(protocol, option),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('protocol', ['s3'])
    @pytest.mark.parametrize('option', OPTION_MAPPING['s3'])
    def test_good_s3(self, id_, occurrence, protocol, option):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(protocol, option),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('protocol', ['sftp'])
    @pytest.mark.parametrize('option', [x for x in OPTIONS_AVAILABLE if x not in OPTION_MAPPING['sftp']])
    def test_bad_sftp(self, id_, occurrence, protocol, option):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(protocol, option),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('protocol', ['sftp'])
    @pytest.mark.parametrize('option', OPTION_MAPPING['sftp'])
    def test_good_sftp(self, id_, occurrence, protocol, option):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(protocol, option),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('protocol', ['npm'])
    @pytest.mark.parametrize('option', [x for x in OPTIONS_AVAILABLE if x not in OPTION_MAPPING['npm']])
    def test_bad_npm(self, id_, occurrence, protocol, option):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(protocol, option),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('protocol', ['npm'])
    @pytest.mark.parametrize('option', OPTION_MAPPING['npm'])
    def test_good_npm(self, id_, occurrence, protocol, option):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(protocol, option),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('protocol', ['npmsw'])
    @pytest.mark.parametrize('option', [x for x in OPTIONS_AVAILABLE if x not in OPTION_MAPPING['npmsw']])
    def test_bad_npmsw(self, id_, occurrence, protocol, option):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(protocol, option),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('protocol', ['npmsw'])
    @pytest.mark.parametrize('option', OPTION_MAPPING['npmsw'])
    def test_good_npmsw(self, id_, occurrence, protocol, option):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(protocol, option),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('protocol', ['svn'])
    @pytest.mark.parametrize('option', [x for x in OPTIONS_AVAILABLE if x not in OPTION_MAPPING['svn']])
    def test_bad_svn(self, id_, occurrence, protocol, option):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(protocol, option),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('protocol', ['svn'])
    @pytest.mark.parametrize('option', OPTION_MAPPING['svn'])
    def test_good_svn(self, id_, occurrence, protocol, option):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(protocol, option),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI += "${@["", "file://init.cfg"][(d.getVar(\'VIRTUAL-RUNTIME_init_manager\') == \'busybox\')]}"',
                                 },
                             ],
                             )
    def test_good_conditionals(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI += "foo.bar.baz"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI[md5sum] = "file://abc"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI[sha256sum] = "file://abc"',
                                 },
                             ],
                             )
    def test_good_edgecases_good(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI += "abc://some.corp.com"',
                                 },
                             ],
                             )
    def test_good_edgecases_bad(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurioptions'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI += "file://kmeta/common;type=kmeta"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI += "git://git.yoctoproject.org/yocto-kernel-cache;type=kmeta;name=meta;branch=yocto-5.10;destsuffix=${KMETA};protocol=foo"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     # nooelint: oelint.vars.srcurioptions
                                     SRC_URI += "\\
                                         file://kmeta/common;type=kmeta \\
                                         abc://some.corp.com \\
                                         foo://some.other.corp \\
                                     "
                                     ''',
                                 },
                             ],
                             )
    def test_type_kmeta_allowed(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintVarAppendOp(TestBaseClass):

    @pytest.mark.parametrize('id_', ['oelint.vars.appendop'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     A ??= "1"
                                     A += "2"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     B += "B"
                                     B ?= "A"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     C ??= "1"
                                     C .= "2"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     D .= "2"
                                     D ?= "1"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     F ??= "1"
                                     F =+ "2"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     G =+ "B"
                                     G ?= "A"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     H ??= "1"
                                     H =. "2"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     I =. "2"
                                     I ?= "1"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     I  =. "2"
                                     I ?= "1"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     G  =+ "B"
                                     G ?= "A"
                                     ''',
                                 },
                             ],
                             )
    def test_bad(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.appendop'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     A ??= "1"
                                     A_append = "2"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     B_append = "B"
                                     B ?= "A"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     C ??= "1"
                                     C_append = "2"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     D_append = "2"
                                     D ?= "1"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     F ??= "1"
                                     F_prepend = "2"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     G_prepend = "B"
                                     G ?= "A"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     H ??= "1"
                                     H_prepend = "2"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     I_prepend = "2"
                                     I ?= "1"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     A ??= "1"
                                     A:append = "2"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     B:append = "B"
                                     B ?= "A"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     C ??= "1"
                                     C:append = "2"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     D:append = "2"
                                     D ?= "1"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     F ??= "1"
                                     F:prepend = "2"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     G:prepend = "B"
                                     G ?= "A"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     H ??= "1"
                                     H:prepend = "2"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     I:prepend = "2"
                                     I ?= "1"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     I:prepend  = "2"
                                     I ?= "1"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     D:append  = "2"
                                     D ?= "1"
                                     ''',
                                 },
                             ],
                             )
    def test_good(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintVarOverride(TestBaseClass):

    @pytest.mark.parametrize('id_', ['oelint.var.override'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     A = "1"
                                     A = "1"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     A = "1"
                                     B = "0"
                                     A = "1"
                                     ''',
                                 },
                             ],
                             )
    def test_bad(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.var.override'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     A = "1"
                                     A += "1"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     A = "1"
                                     A =+ "1"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     A = "1"
                                     A .= "1"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     A = "1"
                                     A  += "1"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     A = "1"
                                     A =. "1"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     A = "1"
                                     A ?= "1"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     A = "1"
                                     A ??= "1"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     A  = "1"
                                     A ??= "1"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     FILESEXTRAPATH_prepend := "a:"
                                     FILESEXTRAPATH_prepend := "b:"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     FILESEXTRAPATH_append := ":a"
                                     FILESEXTRAPATH_append := ":b"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     PACKAGECONFIG = "a b c"
                                     PACKAGECONFIG:remove = "a"
                                     PACKAGECONFIG:remove = "b"
                                     ''',
                                 },
                             ],
                             )
    def test_good(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintVarsVirtual(TestBaseClass):

    @pytest.mark.parametrize('id_', ['oelint.vars.virtual'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     RDEPENDS:${PN} = "virtual/a"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     RPROVIDES:${PN} = "virtual/a"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     require something.inc
                                     RDEPENDS:${PN}-doc = "abc def virtual/a"
                                     ''',
                                 },
                             ],
                             )
    def test_bad(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.dependsappend'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'RDEPENDS:${PN} += "foo"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'RDEPENDS:${PN}:append = "foo"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'RPROVIDES:${PN} += "foo"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'RPROVIDES:${PN}:append = "foo"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'RDEPENDS:${PN}:remove = "virtual/a"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'RPROVIDES:${PN}:remove = "virtual/a"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'RPROVIDES:${PN} += "virtual-reality"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'RPROVIDES:${PN}:append = "virtual-reality"',
                                 },
                             ],
                             )
    def test_good(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintVarDownloadfilename(TestBaseClass):

    @pytest.mark.parametrize('id_', ['oelint.vars.downloadfilename'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI += "http://foo.bar/baz.jpg"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI += "https://foo.bar/baz.jpg"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI += "ftp://foo.bar/baz.jpg"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI += "http://foo.bar/baz.jpg;downloadfilename=something.dat"',
                                 },
                             ],
                             )
    def test_bad(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.var.downloadfilename'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI += "http://foo.bar/baz.jpg;downloadfilename=something.dat.${PV}"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI += "http://foo.bar/baz1.0.jpg"',
                                 },
                                 {
                                     'oelint_adv_test_1.2.3.bb':
                                     'SRC_URI += "http://foo.bar/baz1.2.3.jpg"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI += "git://foo.bar/baz.git"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'SRC_URI += "http://foo.bar/baz.jpg;downloadfilename=something.${PV}"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     PV = "1.2.3"
                                     SRC_URI += "http://foo.bar/baz.jpg;downloadfilename=something.1.2.3"
                                     ''',
                                 },
                             ],
                             )
    def test_good(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)



from typing import List, Tuple

from oelint_parser.cls_item import Variable
from oelint_parser.cls_stash import Stash
from oelint_parser.constants import CONSTANTS

from oelint_adv.cls_rule import Rule


class VarsRenamed(Rule):
    def __init__(self) -> None:
        self._map = CONSTANTS.GetByPath('variables/renamed')
        super().__init__(id='oelint.vars.renamed',
                         severity='error',
                         message='{var} {reason}',
                         valid_from_release='kirkstone')

    def __getMatches(self, _file: str, stash: Stash) -> List[Variable]:
        items: List[Variable] = stash.GetItemsFor(filename=_file,
                                                  classifier=Variable.CLASSIFIER,
                                                  attribute=Variable.ATTR_VAR,
                                                  attributeValue=list(self._map.keys()))
        return items

    def check(self, _file: str, stash: Stash) -> List[Tuple[str, int, str]]:
        res = []
        for i in self.__getMatches(_file, stash):
            message = self._map[i.VarName]
            if ' ' not in message:
                message = f'got renamed to {message}'
            res += self.finding(i.Origin, i.InFileLine,
                                self.Msg.format(var=i.VarName, reason=message))
        return res

    def fix(self, _file: str, stash: Stash) -> List[str]:
        res = []
        for i in self.__getMatches(_file, stash):
            rename = self._map[i.VarName]
            if ' ' not in rename:
                stash.Remove(i)
                i.RealRaw = i.RealRaw.replace(i.VarName, rename)
                i.Raw = i.Raw.replace(i.VarName, rename)
                i = Variable(i.Origin,
                             i.Line,
                             i.InFileLine,
                             i.Raw, rename,
                             i.VarValue,
                             i.VarOp,
                             i.RealRaw, i.IsNewStyleOverrideSyntax)
                stash.Append(i)
                res.append(_file)
        return res



import pytest  # noqa: I900

from .base import TestBaseClass

VAR_ORDER = [
    'SUMMARY',
    'DESCRIPTION',
    'AUTHOR',
    'HOMEPAGE',
    'BUGTRACKER',
    'SECTION',
    'LICENSE',
    'LIC_FILES_CHKSUM',
    'DEPENDS',
    'PROVIDES',
    'PV',
    'SRC_URI',
    'SRCREV',
    'S',
    'inherit',
    'PACKAGECONFIG',
    'EXTRA_QMAKEVARS_POST',
    'EXTRA_OECONF',
    'PACKAGE_ARCH',
    'PACKAGES',
    'FILES_${PN}',
    'RDEPENDS_${PN}',
    'RRECOMMENDS_${PN}',
    'RSUGGESTS_${PN}',
    'RPROVIDES_${PN}',
    'RCONFLICTS_${PN}',
    'BBCLASSEXTEND',
]


class TestClassOelintVarOrder(TestBaseClass):

    def __generate_sample_code(self, first, second):
        return '''
            {first} = "foo"
            {second} = "foo"
            '''.format(first=first, second=second)

    @pytest.mark.parametrize('id_', ['oelint.var.order'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('var', VAR_ORDER)
    def test_bad(self, id_, occurrence, var):
        try:
            id_ += '.{var}'.format(var=var)
            for item in VAR_ORDER[:VAR_ORDER.index(var)]:
                input_ = {
                    'oelint_adv_test.bb': self.__generate_sample_code(item, var),
                }
                self.check_for_id(self._create_args(input_), id_, occurrence)
        except Exception:  # noqa: S110
            pass

    @pytest.mark.parametrize('id_', ['oelint.var.order'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('var', VAR_ORDER)
    def test_good(self, id_, occurrence, var):
        try:
            id_ += '.{var}'.format(var=var)
            for item in VAR_ORDER[VAR_ORDER.index(var):]:
                input_ = {
                    'oelint_adv_test.bb': self.__generate_sample_code(item, var),
                }
                self.check_for_id(self._create_args(input_), id_, occurrence)
        except Exception:  # noqa: S110
            pass

    @pytest.mark.parametrize('id_', ['oelint.var.order.SRCREV'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.inc':
                                     '''
                                     SUMMARY = "..."
                                     DESCRIPTION = "...
                                     AUTHOR = "..."
                                     HOMEPAGE = "..."
                                     BUGTRACKER = "..."
                                     BBCLASSEXTEND += "nativesdk"
                                     ''',
                                     'oelint_adv_test.bb':
                                     '''
                                     require oelint_adv_test.inc
                                     SRCREV = "..."
                                     ''',
                                 },
                             ],
                             )
    def test_single_file_scope(self, id_, occurrence, input_):
        self.check_for_id(self._create_args(input_), id_, occurrence)



from typing import List, Tuple

from oelint_parser.cls_item import Variable
from oelint_parser.cls_stash import Stash

from oelint_adv.cls_rule import Rule


class VarAppendOperation(Rule):
    def __init__(self) -> None:
        super().__init__(id='oelint.vars.appendop',
                         severity='error',
                         message='Use \'{a}\' instead of \'{b}\' as it overwrites \'{c}\'')

    def check(self, _file: str, stash: Stash) -> List[Tuple[str, int, str]]:
        res = []
        items: List[Variable] = stash.GetItemsFor(filename=_file, classifier=Variable.CLASSIFIER)
        for name in {x.VarName for x in items}:
            _weakest = [x for x in items if x.VarName == name and x.VarOp.strip() == '??=']
            _weak = [x for x in items if x.VarName == name and x.VarOp.strip() == '?=']
            _items = [x for x in items if x.VarName == name and x not in _weakest + _weak and x.VarOp.strip() in ['.=', '+=']]
            for i in _items:
                override_delimiter = i.OverrideDelimiter
                if any(_weakest):
                    res += self.finding(i.Origin, i.InFileLine, override_msg=self.Msg.format(
                        a='{od}append'.format(od=override_delimiter), b=i.VarOp, c=_weakest[0].Raw))
                elif any(x.Line > i.Line for x in _weak):
                    res += self.finding(i.Origin, i.InFileLine, override_msg=self.Msg.format(
                        a='{od}append'.format(od=override_delimiter), b=i.VarOp, c=_weak[0].Raw))
            _items = [x for x in items if x.VarName == name and x not in _weakest + _weak and x.VarOp.strip() in ['=.', '=+']]
            for i in _items:
                override_delimiter = i.OverrideDelimiter
                if any(_weakest):
                    res += self.finding(i.Origin, i.InFileLine, override_msg=self.Msg.format(
                        a='{od}prepend'.format(od=override_delimiter), b=i.VarOp, c=_weakest[0].Raw))
                elif any(x.Line > i.Line for x in _weak):
                    res += self.finding(i.Origin, i.InFileLine, override_msg=self.Msg.format(
                        a='{od}prepend'.format(od=override_delimiter), b=i.VarOp, c=_weak[0].Raw))
        return res



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintAppendProtVars(TestBaseClass):

    def __generate_sample_code(self, var, operation):
        return '{var} {operation} "foo"'.format(var=var, operation=operation)

    @pytest.mark.parametrize('id_', ['oelint.append.protvars'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('var', [
        'PV',
        'PR',
        'SRCREV',
        'LICENSE',
        'LIC_FILES_CHKSUM',
    ])
    @pytest.mark.parametrize('operation', ['=', ':=', ' :='])
    def test_bad(self, id_, occurrence, var, operation):
        input_ = {
            'oelint_adv_test.bbappend': self.__generate_sample_code(var, operation),
        }
        id_ += '.{var}'.format(var=var)
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.append.protvars'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('var', [
        'PV',
        'PR',
        'SRCREV',
        'LICENSE',
        'LIC_FILES_CHKSUM',
    ])
    @pytest.mark.parametrize('operation', ['?=', '??=', ' ??='])
    def test_good_weak(self, id_, occurrence, var, operation):
        input_ = {
            'oelint_adv_test.bbappend': self.__generate_sample_code(var, operation),
        }
        id_ += '.{var}'.format(var=var)
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.append.protvars'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('var', [
        'PV',
        'PR',
        'SRCREV',
        'LICENSE',
        'LIC_FILES_CHKSUM',
    ])
    @pytest.mark.parametrize('operation', ['=', ':=', ' :='])
    def test_good_bb(self, id_, occurrence, var, operation):
        input_ = {
            'oelint_adv_test.bb': self.__generate_sample_code(var, operation),
        }
        id_ += '.{var}'.format(var=var)
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.append.protvars'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('var', [
        'PV[vardeps]',
        'PV[vardepsexclude]',
        'PV[vardepvalue]',
        'PV[vardepvalueexclude]',
    ])
    @pytest.mark.parametrize('operation', ['=', ':=', ' :='])
    def test_good_flags(self, id_, occurrence, var, operation):
        input_ = {
            'oelint_adv_test.bbappend': self.__generate_sample_code(var, operation),
        }
        id_ += '.{var}'.format(var=var)
        self.check_for_id(self._create_args(input_), id_, occurrence)



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintFileUnderscores(TestBaseClass):

    @pytest.mark.parametrize('id_', ['oelint.file.underscores'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv-test.bb':
                                     'VAR = "1"',
                                 },
                                 {
                                     'oelintadvtest.bb':
                                     'VAR = "1"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'VAR = "1"',
                                 },
                             ],
                             )
    def test_bad(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.file.underscores'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'inherit core-image',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'inherit image',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'IMAGE_INSTALL += "foo"',
                                 },
                                 {
                                     'oelint-adv_1.2.3.bb':
                                     'VAR = "a"',
                                 },
                                 {
                                     'oelint-adv_git.bb':
                                     'VAR = "a"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'inherit packagegroup',
                                 },
                             ],
                             )
    def test_good(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.file.underscores'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'inherit my-image',
                                 },
                             ],
                             )
    def test_good_image_mod(self, input_, id_, occurrence):
        __cnt = '''
        {
            "images": {
                "known-classes": [
                    "my-image"
                ]
            },
            "functions": {
                "order": ["do_foo"]
            }
        }
        '''
        _extra_opts = [
            '--constantmods=+{mod}'.format(mod=self._create_tempfile('constmod', __cnt))]
        _args = self._create_args(input_, extraopts=_extra_opts)

        self.check_for_id(_args, id, occurrence)



import argparse
import json
import multiprocessing as mp
import os
import sys

# PYTHON_ARGCOMPLETE_OK
import argcomplete

from oelint_adv.cls_rule import load_rules
from oelint_adv.core import TypeSafeAppendAction, arguments_post, parse_configfile, run
from oelint_adv.tweaks import Tweaks
from oelint_adv.version import __version__
from oelint_adv.caches import __default_cache_dir

sys.path.append(os.path.abspath(os.path.join(__file__, '..')))


def _constantmod_completer(prefix, parsed_args, **kwargs):
    if prefix.startswith('+'):  # pragma: no cover
        return [f'+{x}' for x in argcomplete.FilesCompleter().__call__(prefix[1:], **kwargs)]  # pragma: no cover
    return argcomplete.FilesCompleter().__call__(prefix, **kwargs) + ['+', '-']  # pragma: no cover


def create_argparser() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser(prog='oelint-adv',
                                     description='Advanced OELint - Check bitbake recipes against OECore styleguide')
    parser.register('action', 'tsappend', TypeSafeAppendAction)
    parser.add_argument('--suppress', default=[],
                        action='tsappend', help='Rules to suppress')
    parser.add_argument('--output', default=sys.stderr,
                        help='Where to flush the findings (default: stderr)')
    parser.add_argument('--fix', action='store_true', default=False,
                        help='Automatically try to fix the issues')
    parser.add_argument('--nobackup', action='store_true', default=False,
                        help='Don\'t create backup file when auto fixing')
    parser.add_argument('--addrules', nargs='+', default=[],
                        help='Additional non-default rulessets to add')
    parser.add_argument('--customrules', nargs='+', default=[],
                        help='Additional directories to parse for rulessets').completer = argcomplete.DirectoriesCompleter
    parser.add_argument('--rulefile', default=None,
                        help='Rulefile')
    parser.add_argument('--jobs', type=int, default=mp.cpu_count(),
                        help='Number of jobs to run (default all cores)')
    parser.add_argument('--color', action='store_true', default=False,
                        help='Add color to the output based on the severity')
    parser.add_argument('--quiet', action='store_true', default=False,
                        help='Print findings only')
    parser.add_argument('--hide', default=None, action='append',
                        choices=['info', 'warning', 'error'],
                        help='Hide mesesages of specified severity')
    parser.add_argument('--noinfo', action='store_true', default=False,
                        help=argparse.SUPPRESS)
    parser.add_argument('--nowarn', action='store_true', default=False,
                        help=argparse.SUPPRESS)
    parser.add_argument('--mode', choices=['fast', 'all'], default='fast', help='Level of testing (default: fast)')
    parser.add_argument('--relpaths', action='store_true', default=False,
                        help='Show relative paths instead of absolute paths in results')
    parser.add_argument('--messageformat', default='{path}:{line}:{severity}:{id}:{msg}',
                        type=str, help='Format of message output')
    parser.add_argument('--constantmods', default=[], nargs='+',
                        help='''
                             Modifications to the constant db.
                             prefix with:
                             + - to add to DB,
                             - - to remove from DB,
                             None - to override DB
                            ''').completer = _constantmod_completer
    parser.add_argument('--print-rulefile', action='store_true', default=False,
                        help='Print loaded rules as a rulefile and exit')
    parser.add_argument('--exit-zero', action='store_true', default=False,
                        help='Always return a 0 (non-error) status code, even if lint errors are found')
    parser.add_argument('--release', default=Tweaks.DEFAULT_RELEASE, choices=Tweaks.releases(),
                        help='Run against a specific Yocto release')
    parser.add_argument('--cached', action='store_true', help='Use caches (default: off)')
    parser.add_argument('--cachedir', default=os.environ.get('OELINT_CACHE_DIR', __default_cache_dir),
                        help=f'Cache directory (default {__default_cache_dir})')
    parser.add_argument('--clear-caches', action='store_true', help='Clear cache directory and exit')
    # Override the defaults with the values from the config file
    parser.set_defaults(**parse_configfile())

    parser.add_argument(
        'files', nargs='*', help='File to parse').completer = argcomplete.FilesCompleter(allowednames=('bb', 'bbappend', 'bbclass', 'conf'))
    parser.add_argument('--version', action='version',
                        version=f'%(prog)s {__version__}')

    return parser


def parse_arguments() -> argparse.Namespace:
    parser = create_argparser()  # pragma: no cover
    argcomplete.autocomplete(parser)  # pragma: no cover
    return parser.parse_args()  # pragma: no cover


def print_rulefile(args: argparse.Namespace) -> None:
    rules = load_rules(args, add_rules=args.addrules,
                       add_dirs=args.customrules)
    ruleset = {}
    for r in rules:
        ruleset.update(r.get_rulefile_entries())
    print(json.dumps(ruleset, indent=2))  # noqa: T201 - it's here for a reason


def main() -> int:  # pragma: no cover
    args = arguments_post(parse_arguments())

    if args.print_rulefile:
        print_rulefile(args)
        sys.exit(0)

    if args.clear_caches:
        args.state._caches.ClearCaches()
        sys.exit(0)

    try:
        issues = run(args)
    except Exception as e:  # pragma: no cover - that shouldn't be covered anyway
        import traceback
        print('OOPS - That shouldn\'t happen: {e} - {files}'.format(e=e, files=args.files))
        traceback.print_exc()
        # Not using os.EX_SOFTWARE here because it is only available on Unix
        sys.exit(70)

    if args.output != sys.stderr:
        args.output = open(args.output, 'w')
    args.output.write('\n'.join([x[1] for x in issues]))
    if issues:
        args.output.write('\n')
    if args.output != sys.stderr:
        args.output.close()

    # Exit 1 in case of any issue, except when --exit-zero was used.
    exit_code = 0 if args.exit_zero else len(issues) and 1
    sys.exit(exit_code)


if __name__ == '__main__':
    main()  # pragma: no cover



from typing import List, Tuple

from oelint_parser.cls_item import FlagAssignment, Item, Variable
from oelint_parser.cls_stash import Stash
from oelint_parser.parser import INLINE_BLOCK

from oelint_adv.cls_rule import Rule


class VarSRCUriChecksum(Rule):
    def __init__(self) -> None:
        super().__init__(id='oelint.vars.srcurichecksum',
                         severity='error',
                         message='<FOO>')

    def check(self, _file: str, stash: Stash) -> List[Tuple[str, int, str]]:
        res = []
        items: List[Item] = stash.GetItemsFor(filename=_file,
                                              classifier=[FlagAssignment.CLASSIFIER, Variable.CLASSIFIER],
                                              attribute=[FlagAssignment.ATTR_NAME, Variable.ATTR_VAR],
                                              attributeValue="SRC_URI")
        sha256sum = []
        res_candidate = []
        for i in items:
            if isinstance(i, FlagAssignment):
                if i.Flag.endswith("sha256sum"):
                    if i.Flag == "sha256sum":
                        sha256sum.append("")
                    else:
                        sha256sum.append(i.Flag.rsplit(".", 1)[0])
            else:
                lines = [y.strip('"') for y in i.get_items() if y and y != INLINE_BLOCK]
                for index, value in enumerate(lines):
                    _url = stash.GetScrComponents(value)
                    if _url["scheme"] in ["http", "https", "ftp", "ftps", "sftp", "s3", "az"]:
                        name = ""
                        if "name" in _url["options"]:
                            name = _url["options"]["name"]
                        res_candidate.append((name, i.Origin, i.InFileLine + index))

        res_candidate.sort(key=lambda tup: tup[0])

        for (name, filename, filelines) in res_candidate:
            message = ""
            if name == "":
                if "" not in sha256sum:
                    message += "SRC_URI[sha256sum] is needed"
            elif name not in sha256sum:
                message += "SRC_URI[{n}.sha256sum] is needed".format(n=name)
            if len(message) > 0:
                res += self.finding(filename, filelines, message)

        return res



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintVarInherit(TestBaseClass):

    @pytest.mark.parametrize('id_', ['oelint.var.inherit.inherit_defer'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'inherit ${A}',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     inherit def ${C}
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     inherit def ${@oe.utils.ifelse(1 == 1, 'a', C}
                                     ''',
                                 },
                             ],
                             )
    def test_bad_inherit(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_, ['--release=scarthgap']), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.var.inherit.inherit'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'inherit_defer A',
                                 },
                             ],
                             )
    def test_bad_inherit_defer(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_, ['--release=scarthgap']), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.var.inherit.inherit'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     inherit abc def
                                     ''',
                                 },
                             ],
                             )
    def test_good_inherit(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_, ['--release=scarthgap']), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.var.inherit.inherit_defer'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     inherit_defer ${A}
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     inherit_defer native
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     inherit_defer nativesdk
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     inherit_defer cross
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     inherit_defer abc ${A}
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     inherit_defer ${@oe.utils.ifelse(1 == 1, 'a', C}
                                     ''',
                                 },
                             ],
                             )
    def test_good_inherit_defer(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_, ['--release=scarthgap']), id_, occurrence)



from .base import TestBaseClass

# flake8: noqa S101 - n.a. for test files
class TestColor(TestBaseClass):

    def test_get_color_by_severity(self):
        # local scoped import required due PATH manipulation
        from oelint_adv import state
        assert isinstance(state.State().get_color_by_severity('info'), str)

    def test_get_color_by_severity_missing(self):
        # local scoped import required due PATH manipulation
        from oelint_adv import state
        assert state.State().get_color_by_severity('unknown') == ''



from typing import List, Tuple

from oelint_parser.cls_item import FlagAssignment, Variable
from oelint_parser.cls_stash import Stash

from oelint_adv.cls_rule import Rule


class VarSpacesOnAssignment(Rule):
    def __init__(self) -> None:
        super().__init__(id='oelint.vars.spacesassignment',
                         severity='warning',
                         message='Suggest spaces around assignment. E.g. \'FOO = "BAR"\'')

    def check(self, _file: str, stash: Stash) -> List[Tuple[str, int, str]]:
        res = []
        items: List[Variable] = stash.GetItemsFor(filename=_file, classifier=[Variable.CLASSIFIER, FlagAssignment.CLASSIFIER])
        for i in items:
            if i.VarOp not in Variable.VAR_VALID_OPERATOR:
                res += self.finding(i.Origin, i.InFileLine)
        return res
